/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd.
 * kiran-cpanel-system is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 *
 * Author:     yuanxing <yuanxing@kylinos.com.cn>
 */

#include "license-dbus.h"
#include <kiran-log/qt5-log-i.h>
#include <ks-license/license-i.h>
#include <QDBusArgument>
#include <QDBusConnection>
#include <QDBusMessage>
#include <QDBusReply>
#include <QDBusSignature>
#include <QObject>
#include <QtDBus/QDBusReply>
#include <QtDBus/QtDBus>
#include <iostream>

#define KIRAN_LICENSE_INTERFACE "com.kylinsec.Kiran.LicenseHelper"
#define METHOD_GET_LICENSE "GetLicense"
#define METHOD_GET_LICENSE_JSON "GetLicenseJson"
#define METHOD_GET_MACHINE_CODE "GetMachineCode"
#define METHOD_REGISTER_BY_LICENSE_CODE "RegisterByLicenseCode"
#define METHOD_REGISTER_BY_USEBKEY "RegisterByUsbKey"
#define METHOD_REGISTER_ONLINE "RegisterOnLine"
#define METHOD_GETSERVICESTATUS "GetServiceStatus"

#define TIMEOUT_MS 5000
#define TIMEOUT_MS_ONLINE 10000

bool InfoDbus::KylinLicense::getLicenseObject(QString object, QString &LicenseObject)
{
    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(LICENSE_MANAGER_DBUS_NAME,
                                                                LICENSE_MANAGER_OBJECT_PATH,
                                                                LICENSE_MANAGER_DBUS_NAME,
                                                                "GetLicenseObject");
    QList<QVariant> args;
    args.append(QVariant(object));
    msgMethodCall.setArguments(args);

    QDBusMessage msgReply = QDBusConnection::systemBus().call(msgMethodCall,
                                                              QDBus::Block,
                                                              TIMEOUT_MS);

    KLOG_DEBUG() << "msgReply " << msgReply;

    if (msgReply.type() == QDBusMessage::ReplyMessage)
    {
        auto resArgs = msgReply.arguments();
        if (resArgs.size() < 1)
        {
            KLOG_WARNING() << LICENSE_DBUS_NAME << "GetLicenseObject"
                           << "Can't get license object";
        }
        auto objectPath = resArgs.takeFirst();
        QDBusObjectPath *path = (QDBusObjectPath *)objectPath.data();
        LicenseObject = path->path();
        return true;
    }
    if (msgReply.type() == QDBusMessage::ErrorMessage)
    {
        auto errorMsg = msgReply.errorMessage();
        KLOG_WARNING() << LICENSE_DBUS_NAME << "GetLicenseObject"
                       << msgReply.errorName() << msgReply.errorMessage();
        return false;
    }

    return false;
}

bool InfoDbus::KylinLicense::getLicenseJson(QString &licenseInfo, QString object)
{
    QString licenseObject;
    if (!getLicenseObject(object, licenseObject))
    {
        return false;
    }

    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(LICENSE_DBUS_NAME,
                                                                licenseObject,
                                                                LICENSE_OBJECT_DBUS_NAME,
                                                                "GetLicense");

    QDBusMessage msgReply = QDBusConnection::systemBus().call(msgMethodCall, QDBus::Block, TIMEOUT_MS);

    KLOG_DEBUG() << "msgReply " << msgReply;
    QString errMsg;
    if (msgReply.type() == QDBusMessage::ReplyMessage)
    {
        QList<QVariant> args = msgReply.arguments();
        if (args.size() < 1)
        {
            errMsg = "arguments size < 1";
            KLOG_WARNING() << LICENSE_DBUS_NAME << "GetLicense"
                           << msgReply.errorName() << msgReply.errorMessage() << errMsg;
            return false;
        }
        QVariant firstArg = args.takeFirst();
        licenseInfo = firstArg.toString();
        return true;
    }

    return false;
}

bool InfoDbus::KylinLicense::registerByLicenseCode(QString licenseCode, QString &errorMsg, QString object)
{
    QString licenseObject;
    if (!getLicenseObject(object, licenseObject))
    {
        errorMsg = "Can't connect to DBUS interface";
        return false;
    }

    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(LICENSE_DBUS_NAME,
                                                                licenseObject,
                                                                LICENSE_OBJECT_DBUS_NAME,
                                                                "ActivateByActivationCode");
    msgMethodCall << licenseCode;

    QDBusMessage msgReply = QDBusConnection::systemBus().call(msgMethodCall, QDBus::Block, TIMEOUT_MS);

    if (msgReply.type() == QDBusMessage::ErrorMessage)
    {
        errorMsg = msgReply.errorMessage();
        KLOG_WARNING() << LICENSE_DBUS_NAME << "ActivateByActivationCode"
                       << msgReply.errorName() << msgReply.errorMessage();
        return false;
    }

    return true;
}

bool InfoDbus::KylinLicense::registerByUsbKey(QString &points, QString &errorMsg, QString object)
{
    QString licenseObject;
    if (!getLicenseObject(object, licenseObject))
    {
        errorMsg = "Can't connect to DBUS interface";
        return false;
    }
    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(LICENSE_DBUS_NAME,
                                                                licenseObject,
                                                                LICENSE_OBJECT_DBUS_NAME,
                                                                "ActivateByUsbKey");
    QDBusMessage msgReply = QDBusConnection::systemBus().call(msgMethodCall,
                                                              QDBus::Block,
                                                              TIMEOUT_MS);
    KLOG_DEBUG() << "msgReply " << msgReply;

    if (msgReply.type() == QDBusMessage::ReplyMessage)
    {
        QList<QVariant> args = msgReply.arguments();
        if (args.size() < 1)
        {
            errorMsg = "arguments size < 1";
            KLOG_WARNING() << LICENSE_DBUS_NAME << METHOD_REGISTER_BY_USEBKEY
                           << errorMsg;
            return false;
        }
        QVariant firstArg = args.takeFirst();
        points = firstArg.toString();
        return true;
    }
    else if (msgReply.type() == QDBusMessage::ErrorMessage)
    {
        errorMsg = msgReply.errorMessage();
        KLOG_WARNING() << LICENSE_DBUS_NAME << METHOD_REGISTER_BY_USEBKEY
                       << msgReply.errorName() << msgReply.errorMessage();
        return false;
    }
    return false;
}

bool InfoDbus::KylinLicense::registerOnline(QString ip, QString &errorMsg, QString &pointsUid, QString object)
{
    QString licenseObject;
    if (!getLicenseObject(object, licenseObject))
    {
        errorMsg = "Can't connect to DBUS interface";
        return false;
    }
    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(LICENSE_DBUS_NAME,
                                                                licenseObject,
                                                                LICENSE_OBJECT_DBUS_NAME,
                                                                "ActivateOnLine");
    msgMethodCall << ip;
    msgMethodCall << pointsUid;

    QDBusMessage msgReply = QDBusConnection::systemBus().call(msgMethodCall,
                                                              QDBus::Block,
                                                              TIMEOUT_MS_ONLINE);

    KLOG_DEBUG() << "msgReply " << msgReply;

    QDBusReply<void> reply = QDBusReply<void>(msgReply);
    if (!reply.isValid())
    {
        QDBusError error = reply.error();
        if (error.type() == QDBusError::NoReply)
        {
            errorMsg = QObject::tr("Did not reply within the specified timeout");
            goto failed;
        }
        else if (error.type() == QDBusError::ServiceUnknown)
        {
            errorMsg = QObject::tr("The called service is not known");
            goto failed;
        }
        else if (msgReply.type() == QDBusMessage::ErrorMessage)
        {
            errorMsg = msgReply.errorMessage();
            goto failed;
        }
    }
    return true;

failed:
    KLOG_WARNING() << LICENSE_DBUS_NAME << METHOD_REGISTER_ONLINE
                   << msgReply.errorName() << msgReply.errorMessage();
    return false;
}

// bool InfoDbus::KylinLicense::getServiceStatus(int &status)
//{
//     QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(LICENSE_DBUS_NAME,
//                                                                 LICENSE_DBUS_OBJECT_PATH,
//                                                                 KIRAN_LICENSE_INTERFACE,
//                                                                 METHOD_GETSERVICESTATUS);

//    QDBusMessage msgReply = QDBusConnection::systemBus().call(msgMethodCall,
//                                                              QDBus::Block,
//                                                              TIMEOUT_MS);
//    KLOG_DEBUG() << "msgReply " << msgReply;
//    QString errMsg;
//    if (msgReply.type() == QDBusMessage::ReplyMessage)
//    {
//        QList<QVariant> args = msgReply.arguments();
//        if (args.size() < 1)
//        {
//            errMsg = "arguments size < 1";
//            goto failed;
//        }
//        QVariant firstArg = args.takeFirst();
//        status = firstArg.toInt();
//        return true;
//    }
// failed:
//    KLOG_WARNING() << LICENSE_DBUS_NAME << METHOD_GETSERVICESTATUS
//                   << msgReply.errorName() << msgReply.errorMessage() << errMsg;
//    return false;
//}
