#include "notify-base.h"
#include <json/json.h>
#include <lib/dbus/license-dbus.h>
#include <QString>
#include <QtDBus>
#include <QtMath>
#include <memory>
#include "kiran-log/qt5-log-i.h"
#include "ks-license/license-i.h"
#include "notification-wrapper.h"

namespace Notify
{
#define NOTIFY_NORMA_INDEX -1
NotifyBase::NotifyBase(QObject *parent) : QObject(parent)
{
    m_licenseCheckTimer = new QTimer(this);
    m_loopNotifyTimer = new QTimer(this);
    m_checkLicenseTimes = 0;
}

void NotifyBase::run()
{
    // 开启license循环检测
    m_licenseCheckTimer->start(30000);
    connect(m_licenseCheckTimer, SIGNAL(timeout()), this, SLOT(timingCheckLicense()));
    connect(this, SIGNAL(sigActiveNotify(qint64, QString)), this, SLOT(notifyActive(qint64, QString)));
    QDBusConnection::sessionBus().connect(QString(),
                                          KIRAN_SCREENSAVER_DBUS_PATH,
                                          KIRAN_SCREENSAVER_DBUS_INTERFACE,
                                          "ActiveChanged",
                                          this,
                                          SLOT(onActiveChange(bool)));
    QDBusConnection::sessionBus().connect(QString(),
                                          MATE_SCREENSAVER_DBUS_PATH,
                                          MATE_SCREENSAVER_DBUS_INTERFACE,
                                          "ActiveChanged",
                                          this,
                                          SLOT(onActiveChange(bool)));
    this->timingCheckLicense();
}

bool NotifyBase::insertNotifyType(QMap<int64_t, NotifyType> &stateMap,
                                  int64_t flagDay,
                                  QString notifyName,
                                  bool loopNotify,
                                  int loopSecs)
{
    NotifyType notifyType;
    notifyType.notifyName = notifyName;
    notifyType.loopNotify = loopNotify;
    notifyType.loopSecs = loopSecs;

    if (!stateMap.count(flagDay))
    {
        stateMap.insert(flagDay, notifyType);
        return true;
    }
    else
    {
        KLOG_WARNING() << "The same flagDay: " << flagDay << "exists";
        return false;
    }
}

bool NotifyBase::registerNotify(bool licenseState,
                                int64_t flagDay,
                                QString notifyName,
                                bool loopNotify,
                                int loopSecs)
{
    if (licenseState)
    {
        return insertNotifyType(m_activedTypes, flagDay, notifyName, loopNotify, loopSecs);
    }
    else
    {
        return insertNotifyType(m_inactiveTypes, flagDay, notifyName, loopNotify, loopSecs);
    }
}

void NotifyBase::registerNormalNotify(bool licenseState,
                                      QString notifyName,
                                      bool loopNotify,
                                      int loopSecs)
{
    if (licenseState)
    {
        m_activedNormalType.notifyName = notifyName;
        m_activedNormalType.loopNotify = loopNotify;
        m_activedNormalType.loopSecs = loopSecs;
    }
    else
    {
        m_inactiveNormalType.notifyName = notifyName;
        m_inactiveNormalType.loopNotify = loopNotify;
        m_inactiveNormalType.loopSecs = loopSecs;
    }
}

LicenseInfo NotifyBase::getLicense()
{
    QString licenseStr;
    LicenseInfo licenseInfo;

    if (!InfoDbus::KylinLicense::getLicenseJson(licenseStr, LICENSE_OBJECT_OS_NAME))
    {
        KLOG_WARNING("can't connect to DBUS interface");
        return licenseInfo;
    }

    QJsonParseError jsonError;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(licenseStr.toLocal8Bit().data(), &jsonError);
    if (jsonDocument.isNull() || jsonError.error != QJsonParseError::NoError)
    {
        KLOG_DEBUG() << " please check the activation information string " << licenseStr.toLocal8Bit().data();
        return licenseInfo;
    }
    if (jsonDocument.isObject())
    {
        QJsonObject obj = jsonDocument.object();
        if (obj.contains(LICENSE_JK_ACTIVATION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_STATUS);
            if (value.isDouble())
            {
                licenseInfo.licensed = value.toVariant().toInt();
                KLOG_INFO() << "activation status :" << licenseInfo.licensed;
            }
        }
        if (obj.contains(LICENSE_JK_ACTIVATION_CODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_CODE);
            if (value.isString())
            {
                licenseInfo.activationCode = value.toVariant().toString();
                KLOG_INFO() << "lc_code :" << licenseInfo.activationCode;
            }
        }
        if (obj.contains(LICENSE_JK_MACHINE_CODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_MACHINE_CODE);
            if (value.isString())
            {
                licenseInfo.machineCode = value.toVariant().toString();
                KLOG_INFO() << "mc_code :" << licenseInfo.machineCode;
            }
        }
        if (obj.contains(LICENSE_JK_EXPIRED_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXPIRED_TIME);
            if (value.isDouble())
            {
                licenseInfo.expiredTime = time_t(value.toVariant().toUInt());
                KLOG_INFO() << "expired_time :" << licenseInfo.expiredTime;
            }
        }
        if (obj.contains(LICENSE_JK_FIRST_BOOT_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_FIRST_BOOT_TIME);
            if (value.isDouble())
            {
                licenseInfo.firstBootTime = value.toVariant().toInt();
                KLOG_INFO() << "first boot time :" << licenseInfo.firstBootTime;
            }
        }
        if (obj.contains(LICENSE_JK_SERVICE_MODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_SERVICE_MODE);
            if (value.isDouble())
            {
                licenseInfo.serviceMode = value.toVariant().toInt();
                KLOG_INFO() << "service mode :" << licenseInfo.serviceMode;
            }
        }
        if (obj.contains(LICENSE_JK_UPDATE_MODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_UPDATE_MODE);
            if (value.isDouble())
            {
                licenseInfo.updateMode = value.toVariant().toInt();
                KLOG_INFO() << "update mode :" << licenseInfo.updateMode;
            }
        }
        if (obj.contains(LICENSE_JK_EXCEPTION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXCEPTION_STATUS);
            if (value.isDouble())
            {
                licenseInfo.exceptionStatus = value.toVariant().toInt();
                KLOG_INFO() << "exception status:" << licenseInfo.exceptionStatus;
            }
        }
    }

    return licenseInfo;
}

void NotifyBase::setLoopNotify(QString notifyName,
                               int loopSecs)
{
    if (m_loopNotifyTimer->isActive() && m_currentNotifyName != notifyName)
    {
        m_loopNotifyTimer->stop();
    }
    else if (m_loopNotifyTimer->isActive() && m_currentNotifyName == notifyName)
    {
        return;
    }

    m_loopNotifyTimer->setInterval(loopSecs * 1000);
    connect(m_loopNotifyTimer, SIGNAL(timeout()), this, SLOT(loopNotify()));
    m_loopNotifyTimer->start();
    m_currentNotifyName = notifyName;
    this->loopNotify();
}

void NotifyBase::timingCheckLicense()
{
    m_checkLicenseTimes++;

    m_licenseInfo = getLicense();

    auto currentTime = QDateTime::currentDateTime();
    auto notifyStatus = m_licenseInfo.licensed == LicenseActivationStatus::LAS_ACTIVATED ? m_activedTypes : m_inactiveTypes;
    auto normalNotifyType = m_licenseInfo.licensed == LicenseActivationStatus::LAS_ACTIVATED ? m_activedNormalType : m_inactiveNormalType;
    auto flagDays = notifyStatus.keys();

    // 查找是否存在标记日期小于剩余质保天数
    int index = NOTIFY_NORMA_INDEX;
    m_remainLicenseDays = currentTime.daysTo(QDateTime::fromTime_t(m_licenseInfo.expiredTime));

    for (int i = 0; i < flagDays.size(); i++)
    {
        if (m_remainLicenseDays < flagDays[i])
        {
            index = i;
            break;
        }
    }

    // 根据标记发出信号
    if (index == NOTIFY_NORMA_INDEX)
    {
        if (normalNotifyType.loopNotify)
        {
            setLoopNotify(normalNotifyType.notifyName,
                          normalNotifyType.loopSecs);
        }
        else
        {
            m_currentNotifyName = normalNotifyType.notifyName;
            m_loopNotifyTimer->stop();
            if (m_checkLicenseTimes == 1)
            {
                emit sigActiveNotify(m_remainLicenseDays, normalNotifyType.notifyName);
                KLOG_DEBUG() << "Current notify: " << m_currentNotifyName;
            }
        }
    }
    else
    {
        if (notifyStatus[flagDays[index]].loopNotify)
        {
            setLoopNotify(notifyStatus[flagDays[index]].notifyName,
                          notifyStatus[flagDays[index]].loopSecs);
        }
        else
        {
            m_currentNotifyName = notifyStatus[flagDays[index]].notifyName;
            m_loopNotifyTimer->stop();
            if (m_checkLicenseTimes == 1)
            {
                emit sigActiveNotify(m_remainLicenseDays, notifyStatus[flagDays[index]].notifyName);
                KLOG_DEBUG() << "Current notify: " << m_currentNotifyName;
            }
        }
    }
}

void NotifyBase::loopNotify()
{
    emit sigActiveNotify(m_remainLicenseDays, m_currentNotifyName);
    KLOG_DEBUG() << "Current notify: " << m_currentNotifyName;
}

void NotifyBase::onActiveChange(bool state)
{
    // FIXME: 在主版本中， m_loopNotifyTimer 的间隔时间未被初始化，导致间隔时间为0, 一直占用大量 cpu 资源，所以在此处判断间隔时间为0则直接返回， 需要修复。(#13240)
    if (m_loopNotifyTimer->interval() == 0)
    {
        return;
    }
    KLOG_DEBUG() << "screen state change" << state;
    if (state)
    {
        m_loopNotifyTimer->stop();
    }
    else
    {
        m_loopNotifyTimer->start();
    }
}

}  // namespace Notify
