#include "service-info-table.h"
#include <QFileDialog>
#include "kiran-message-box.h"
#include "ks-license/license-i.h"
#include "ui_service-info-table.h"

enum PointsTableHoriz
{
    PointsNumber,
    PointsUid,
    PointsCustomerInfo,
    PointsContractNo,
    PointsProjectName,
    PointsActivationObjectName,
    PointsActivationObjectNotes,
    PointsExpiryDate,
    PointsInstallType,
    PointsUpdateMode,
    PointsServiceMode,
    PointsSumPoints,
    PointsRemainingPoints,
    // PointsSumRemain,
    PointsMax
};

ServiceInfoTable::ServiceInfoTable(QWidget *parent) : QWidget(parent),
                                                      ui(new Ui::ServiceInfoTable)
{
    ui->setupUi(this);
    m_serverGrpc = LicenseGrpcThread::getLicenseGrpc();

    connect(this,
            SIGNAL(getPointsRecords()),
            m_serverGrpc, SLOT(onGetPointsRecords()),
            Qt::QueuedConnection);

    connect(this,
            SIGNAL(importPointsRecords(QString)),
            m_serverGrpc, SLOT(onImportPointsRecords(QString)),
            Qt::QueuedConnection);

    connect(m_serverGrpc,
            SIGNAL(getPointsRecordsRsp(QList<Point>, bool)),
            this,
            SLOT(onGetRecords(QList<Point>, bool)),
            Qt::BlockingQueuedConnection);

    connect(m_serverGrpc,
            SIGNAL(importPointsRecordsRsp(QString)),
            this, SLOT(onImportPointsRecords(QString)),
            Qt::BlockingQueuedConnection);

    connect(ui->lineEdit_points_search,
            SIGNAL(returnPressed()),
            this,
            SLOT(on_pushButton_points_search_clicked()));

    initUI();
}

ServiceInfoTable::~ServiceInfoTable()
{
    delete ui;
}

void ServiceInfoTable::resizeEvent(QResizeEvent *event)
{
    updateTableHeader();
}

void ServiceInfoTable::initUI()
{
    initPointsTable();

    ui->pushButton_points_search->setIcon(QIcon(":/ksl-search.svg"));
    ui->lineEdit_points_search->setPlaceholderText("Search...");

    ui->widget_points_search->setProperty("editfocus", false);
    ui->lineEdit_points_search->installEventFilter(this);
    ui->pushButton_points_search->installEventFilter(this);
    style()->polish(this);
}

void ServiceInfoTable::onGetRecords(QList<Point> pointsData, bool res)
{
    if (res)
    {
        updateTable(pointsData);
    }
    else
    {
        KiranMessageBox::message(this,
                                 QObject::tr("warning"),
                                 QObject::tr("Unable connet to Server"),
                                 KiranMessageBox::Ok);
    }
}

void ServiceInfoTable::updateTableHeader()
{
    int tableWidth = ui->tableView_points_info->width();
    int cellWidth = tableWidth / 23;
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsNumber, 1 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsUid, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsCustomerInfo, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsContractNo, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsProjectName, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsActivationObjectName, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsExpiryDate, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsInstallType, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsUpdateMode, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsServiceMode, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsSumPoints, 2 * cellWidth);
    ui->tableView_points_info->horizontalHeader()->resizeSection(PointsRemainingPoints, 2 * cellWidth);
}

void ServiceInfoTable::updateTable(QList<Point> pointDataList)
{
    m_tableModel->removeRows(0, m_tableModel->rowCount());
    for (int i = 0; i < pointDataList.length(); i++)
    {
        setpointItem(pointDataList[i]);
    }
}

void ServiceInfoTable::setpointItem(Point pointData)
{
    QStringList pointsDataStrList;
    int row = m_tableModel->rowCount();
    ui->tableView_points_info->setRowHeight(row + 1, 50);
    QString expiryTime;
    if (pointData.computeExpiryDayMode == ComputeExpiryDayMode::COMPUTE_EXPIRY_DAY_BY_ACTIVATE)
    {
        if (pointData.warrantyYears > 0)
        {
            expiryTime += QString::number(pointData.warrantyYears) + tr(" Years ");
        }
        if (pointData.warrantyMonths > 0)
        {
            expiryTime += QString::number(pointData.warrantyMonths) + tr(" Months ");
        }
        if (pointData.warrantyDays > 0)
        {
            expiryTime += QString::number(pointData.warrantyDays) + tr(" Days ");
        }
    }
    else
    {
        expiryTime = pointData.expiryDate;
    }
    pointsDataStrList << pointData.uid
                      << pointData.customerInfo
                      << pointData.contractNo
                      << pointData.projectName
                      << pointData.activationObjectName +
                             (pointData.activationObjectNotes.isEmpty() ? "" : " (" + pointData.activationObjectNotes + ")")
                      << expiryTime
                      << installType2Str(pointData.installType)
                      << updateMode2Str(pointData.updateMode)
                      << serviceMode2Str(pointData.serviceMode)
                      << QString("%1").arg(pointData.sumPoints)
                      << QString("%1").arg(pointData.remainingPoints);

    foreach (QString data, pointsDataStrList)
    {
        if (data.contains(m_searchKeywords))
        {
            m_tableModel->setItem(row, PointsNumber, new QStandardItem(QString("%1").arg(row + 1)));
            for (int i = 0; i < pointsDataStrList.length(); i++)
            {
                if (pointsDataStrList[i] == "")
                {
                    pointsDataStrList[i] = "-";
                }
                m_tableModel->setItem(row, i + 1, new QStandardItem(pointsDataStrList[i]));
            }
            break;
        }
    }
}

void ServiceInfoTable::on_pushButton_points_import_clicked()
{
    QString dbFileName = QFileDialog::getOpenFileName(this, QObject::tr("Choose file"), QDir::homePath(), "*.db");
    if (dbFileName != "")
    {
        emit importPointsRecords(dbFileName);
    }
}

void ServiceInfoTable::on_pushButton_points_search_clicked()
{
    m_searchKeywords = ui->lineEdit_points_search->text();
    emit getPointsRecords();
}

void ServiceInfoTable::onImportPointsRecords(QString error)
{
    if (error.isEmpty())
    {
        KiranMessageBox::message(this,
                                 QObject::tr("Succeed!"),
                                 QObject::tr("Import Points Succeed!"),
                                 KiranMessageBox::Ok);
        emit getPointsRecords();
    }
    else
    {
        KiranMessageBox::message(this,
                                 QObject::tr("Failed!"),
                                 error,
                                 KiranMessageBox::Ok);
    }
}

void ServiceInfoTable::onItemDoubleClicked(const QModelIndex &index)
{
    int row = index.row();
    QStandardItem *item = m_tableModel->item(row, PointsUid);
    QString pointsUid = item->text();
    emit doubleClikedService(pointsUid);
}

void ServiceInfoTable::initPointsTable()
{
    m_tableModel = new QStandardItemModel(this);
    ui->tableView_points_info->setModel(m_tableModel);
    connect(ui->tableView_points_info, SIGNAL(doubleClicked(const QModelIndex &)),
            this, SLOT(onItemDoubleClicked(const QModelIndex &)));
    m_tableModel->setHorizontalHeaderLabels(QStringList() << QObject::tr("Number")
                                                          << QObject::tr("ServiceUid")
                                                          << QObject::tr("CustomerInfo")
                                                          << QObject::tr("ContractNo")
                                                          << QObject::tr("ProjectName")
                                                          << QObject::tr("ActiveObject")
                                                          << QObject::tr("ExpiryDate")
                                                          << QObject::tr("InstallStates")
                                                          << QObject::tr("UpdateMode")
                                                          << QObject::tr("ServiceMode")
                                                          << QObject::tr("SumPoints")
                                                          << QObject::tr("Remain"));

    initTableBase(ui->tableView_points_info, "pointsInfoHeader");
    updateTableHeader();
    emit getPointsRecords();
}

void ServiceInfoTable::initTableBase(QTableView *tableView, const QString &tableName)
{
    tableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
    tableView->horizontalHeader()->setObjectName(tableName);
    tableView->verticalHeader()->hide();
    tableView->setAlternatingRowColors(true);
    tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    tableView->setShowGrid(false);
    tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    tableView->horizontalHeader()->setStretchLastSection(true);
    tableView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft | Qt::AlignVCenter);
}

bool ServiceInfoTable::eventFilter(QObject *watched, QEvent *event)
{
    if (watched == ui->lineEdit_points_search || ui->pushButton_points_search)
    {
        if (event->type() == QEvent::FocusIn)
        {
            ui->widget_points_search->setProperty("editfocus", true);
            style()->unpolish(ui->widget_points_search);
            style()->polish(ui->widget_points_search);
            ui->widget_points_search->update();
        }
        if (event->type() == QEvent::FocusOut)
        {
            ui->widget_points_search->setProperty("editfocus", false);
            style()->unpolish(ui->widget_points_search);
            style()->polish(ui->widget_points_search);
            ui->widget_points_search->update();
        }
    }

    return QWidget::eventFilter(watched, event);
}

void ServiceInfoTable::onActiveCurrent()
{
    updateTableHeader();
    emit getPointsRecords();
}

QString ServiceInfoTable::installType2Str(int installType)
{
    QString installTypeString;
    switch (installType)
    {
    case LICENSE_INSTALL_TYPE_OLD_DESKTOP:
    case LICENSE_INSTALL_TYPE_DESKTOP:
        installTypeString = QObject::tr("Desktop");
        break;
    case LICENSE_INSTALL_TYPE_OLD_SERVER:
    case LICENSE_INSTALL_TYPE_SERVER:
        installTypeString = QObject::tr("Server");
        break;
    case LICENSE_INSTALL_TYPE_OLD_WORKSTATION:
        installTypeString = QObject::tr("Workstation");
        break;
    case LICENSE_INSTALL_TYPE_OLD_INDUSTRIAL:
    case LICENSE_INSTALL_TYPE_OLD_LAST:
    case LICENSE_INSTALL_TYPE_INDUSTRIAL:
        installTypeString = QObject::tr("Industrial");
        break;
    case LICENSE_INSTALL_TYPE_UNLIMIT:
        installTypeString = QObject::tr("Unlimit");
        break;
    case LICENSE_INSTALL_TYPE_DEVELOPMENT:
    case LICENSE_INSTALL_TYPE_LAST:
        installTypeString = QObject::tr("Development");
        break;
    default:
        installTypeString = QObject::tr("Unknow");
    }
    return installTypeString;
}

QString ServiceInfoTable::updateMode2Str(int updateMode)
{
    QString updateModeString;
    switch (updateMode)
    {
    case LICENSE_UPDATE_MODE_NONE:
        updateModeString = "-";
        break;
    case LICENSE_UPDATE_MODE_PERMANENT:
        updateModeString = QObject::tr("Permanent");
        break;
    case LICENSE_UPDATE_MODE_SUBSCRIBE:
        updateModeString = QObject::tr("Subscribe");
        break;
    default:
        updateModeString = QObject::tr("Unknow");
    }
    return updateModeString;
}

QString ServiceInfoTable::serviceMode2Str(int serviceMode)
{
    QString serviceModeString;
    switch (serviceMode)
    {
    case LICENSE_SERVICE_MODE_NONE:
        serviceModeString = " - ";
        break;
    case LICENSE_SERVICE_MODE_7X24:
        serviceModeString = QObject::tr("7x24 hours");
        break;
    case LICENSE_SERVICE_MODE_5X8:
        serviceModeString = QObject::tr("5x8 hours");
        break;
    default:
        serviceModeString = QObject::tr("Unknow");
    }
    return serviceModeString;
}
