#include "active-guide.h"
#include <QDBusObjectPath>
#include <QJsonDocument>
#include <QJsonObject>
#include <QMessageBox>
#include <QScreen>
#include <QtDBus>
#include "cmake-define.h"
#include "dbus/license-dbus.h"
#include "kiran-log/qt5-log-i.h"
#include "ui_active-guide.h"

namespace VA
{
#define DBUS_TIMEOUT_MS 5000

ActiveGuide::ActiveGuide(const QString &object, const QIcon &icon, QWidget *parent) : QWidget(parent),
                                                                                      ui(new Ui::ActiveGuide),
                                                                                      m_object(object),
                                                                                      m_reactiveFlag(false)
{
    ui->setupUi(this);

    ui->pushButton_QRCode->installEventFilter(this);
    m_dialog = new Dialog(icon, this);
    m_showQRCode = new ShowQRCode(tr("Machine code"), icon);
    m_licenseInfo = getLicenseInfo();
    connect(this, SIGNAL(sig_showQRCodeWgt(QPoint, QObject *)), this, SLOT(popupQRCode(QPoint, QObject *)));
    initUi();
}

ActiveGuide::~ActiveGuide()
{
    delete ui;
    delete m_showQRCode;
}

void ActiveGuide::initUi()
{
    ui->label_MCCodeShow->setText(m_licenseInfo.machineCode);
    ui->widget_ExpiryDay->hide();

    if (m_licenseInfo.licensed == LicenseActivationStatus::LAS_ACTIVATED)
    {
        ui->lineEdit_ACCode->setText(m_licenseInfo.activationCode);
        ui->lineEdit_ACCode->setDisabled(true);
        ui->lineEdit_ACCode->setStyleSheet("QLineEdit#lineEdit_ACCode{color:#919191; font-size: 14px; background-color: #393939;"
                                           "border-radius: 6px;border:1px solid #393939;padding-left:10px;}"
                                           "QLineEdit#lineEdit_ACCode:focus{border:1px solid #2eb3ff}");
        ui->label_ExpiryDayShow->setText(QDateTime::fromTime_t(m_licenseInfo.expiredTime).toString("yyyy-MM-dd"));
        ui->widget_ExpiryDay->show();
        ui->pushButton_Active->setText(tr("Reactive"));
    }
    else if (m_licenseInfo.licensed == LicenseActivationStatus::LAS_UNACTIVATED)
    {
        ui->lineEdit_ACCode->setStyleSheet("QLineEdit#lineEdit_ACCode{color:#ffffff; font-size: 14px; background-color: #222222;"
                                           "border-radius: 6px;border:1px solid #222222;padding-left:10px;}"
                                           "QLineEdit#lineEdit_ACCode:focus{border:1px solid #2eb3ff}");
    }
    setMaximumSize(632, 316);
    setMinimumSize(632, 316);
}

LicenseInfo ActiveGuide::getLicenseInfo()
{
    QString licenseStr;
    LicenseInfo licenseInfo;

    if (!InfoDbus::KylinLicense::getLicenseJson(licenseStr, m_object))
    {
        KLOG_WARNING("can't connect to DBUS interface");
        displayMessageBox(QObject::tr("error"), QObject::tr("Can't connect to DBUS interface"));
        emit sigClose();
        return licenseInfo;
    }

    QJsonParseError jsonError;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(licenseStr.toLocal8Bit().data(), &jsonError);
    if (jsonDocument.isNull() || jsonError.error != QJsonParseError::NoError)
    {
        KLOG_DEBUG() << " please check the activation information string " << licenseStr.toLocal8Bit().data();
        return licenseInfo;
    }
    if (jsonDocument.isObject())
    {
        QJsonObject obj = jsonDocument.object();
        if (obj.contains(LICENSE_JK_ACTIVATION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_STATUS);
            if (value.isDouble())
            {
                licenseInfo.licensed = value.toVariant().toInt();
                KLOG_INFO() << "activation status :" << licenseInfo.licensed;
            }
        }
        if (obj.contains(LICENSE_JK_ACTIVATION_CODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_CODE);
            if (value.isString())
            {
                licenseInfo.activationCode = value.toVariant().toString();
                KLOG_INFO() << "lc_code :" << licenseInfo.activationCode;
            }
        }
        if (obj.contains(LICENSE_JK_MACHINE_CODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_MACHINE_CODE);
            if (value.isString())
            {
                licenseInfo.machineCode = value.toVariant().toString();
                KLOG_INFO() << "mc_code :" << licenseInfo.machineCode;
            }
        }
        if (obj.contains(LICENSE_JK_EXPIRED_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXPIRED_TIME);
            if (value.isDouble())
            {
                licenseInfo.expiredTime = time_t(value.toVariant().toUInt());
                KLOG_INFO() << "expired_time :" << licenseInfo.expiredTime;
            }
        }
        if (obj.contains(LICENSE_JK_FIRST_BOOT_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_FIRST_BOOT_TIME);
            if (value.isDouble())
            {
                licenseInfo.firstBootTime = value.toVariant().toInt();
                KLOG_INFO() << "first boot time :" << licenseInfo.firstBootTime;
            }
        }
        if (obj.contains(LICENSE_JK_SERVICE_MODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_SERVICE_MODE);
            if (value.isDouble())
            {
                licenseInfo.serviceMode = value.toVariant().toInt();
                KLOG_INFO() << "service mode :" << licenseInfo.serviceMode;
            }
        }
        if (obj.contains(LICENSE_JK_UPDATE_MODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_UPDATE_MODE);
            if (value.isDouble())
            {
                licenseInfo.updateMode = value.toVariant().toInt();
                KLOG_INFO() << "update mode :" << licenseInfo.updateMode;
            }
        }
        if (obj.contains(LICENSE_JK_EXCEPTION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXCEPTION_STATUS);
            if (value.isDouble())
            {
                licenseInfo.exceptionStatus = value.toVariant().toInt();
                KLOG_INFO() << "exception status:" << licenseInfo.exceptionStatus;
            }
        }
    }

    return licenseInfo;
}

void ActiveGuide::on_pushButton_Active_clicked()
{
    if (tr("Reactive") == ui->pushButton_Active->text())
    {
        ui->lineEdit_ACCode->setEnabled(true);
        ui->lineEdit_ACCode->clear();
        ui->pushButton_Active->setText(tr("Active"));
        ui->lineEdit_ACCode->setStyleSheet("QLineEdit#lineEdit_ACCode{color:#ffffff; font-size: 14px; background-color: #222222;"
                                           "border-radius: 6px;border:1px solid #222222;padding-left:10px;}"
                                           "QLineEdit#lineEdit_ACCode:focus{border:1px solid #2eb3ff}");
        m_reactiveFlag = true;
        ui->widget_ExpiryDay->hide();
        return;
    }

    if (ui->lineEdit_ACCode->text().isEmpty())
    {
        displayMessageBox(QObject::tr("waring"), QObject::tr("Please input activation code!"));
        return;
    }

    QString errorMsg;
    if (!InfoDbus::KylinLicense::registerByLicenseCode(ui->lineEdit_ACCode->text(), errorMsg, m_object))
    {
        displayMessageBox(QObject::tr("error"), errorMsg);
    }
    else
    {
        displayMessageBox(QObject::tr("success"), QObject::tr("Active success!"));
        ui->lineEdit_ACCode->setEnabled(false);
        ui->pushButton_Active->setText(tr("Reactive"));
        ui->lineEdit_ACCode->setStyleSheet("QLineEdit#lineEdit_ACCode{color:#919191; font-size: 14px; background-color: #393939;"
                                           "border-radius: 6px;border:1px solid #393939;padding-left:10px;}");

        m_licenseInfo = getLicenseInfo();
        ui->label_ExpiryDayShow->setText(QDateTime::fromTime_t(m_licenseInfo.expiredTime).toString("yyyy-MM-dd"));
        ui->widget_ExpiryDay->show();
        m_reactiveFlag = false;
    }
}

void ActiveGuide::on_pushButton_Cancel_clicked()
{
    if (m_reactiveFlag)
    {
        ui->lineEdit_ACCode->setEnabled(false);
        ui->lineEdit_ACCode->setText(m_licenseInfo.activationCode);
        ui->pushButton_Active->setText(tr("Reactive"));
        ui->lineEdit_ACCode->setStyleSheet("QLineEdit#lineEdit_ACCode{color:#919191; font-size: 14px; background-color: #393939;"
                                           "border-radius: 6px;border:1px solid #393939;padding-left:10px;}");

        m_licenseInfo = getLicenseInfo();
        ui->label_ExpiryDayShow->setText(QDateTime::fromTime_t(m_licenseInfo.expiredTime).toString("yyyy-MM-dd"));
        ui->widget_ExpiryDay->show();
        m_reactiveFlag = false;
        return;
    }
    emit sigClose();
}

/**
 * @brief  事件监听，事件监听，监听鼠标进入或离开二维码按钮事件 ，
 *         进入按钮则发出弹出二维码信号，离开则隐藏二维码窗口，并释放二维码窗口内存
 * @param  obj  事件对象
 * @param  obj  事件
 * @return 是否过滤
 */
bool ActiveGuide::eventFilter(QObject *target, QEvent *e)
{
    if (target == ui->pushButton_QRCode)
    {
        if (e->type() == QEvent::MouseButtonPress)
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(e);
            if (target == ui->pushButton_QRCode)
            {
                QPoint oPoint = QPoint(mouseEvent->globalX() - 150, mouseEvent->globalY() - 36);
                emit sig_showQRCodeWgt(oPoint, target);
            }
            else
            {
                QPoint oPoint = QPoint(mouseEvent->globalX() - 150, mouseEvent->globalY() - 36);
                emit sig_showQRCodeWgt(oPoint, target);
            }
        }
    }
    return QWidget::eventFilter(target, e);
}

/**
 * @brief  显示二维码的回调函数
 * @param  oPoint  二维码显示的位置
 * @param  target  事件对象，用来决定显示机器码还是二维码
 */
void ActiveGuide::popupQRCode(QPoint oPoint, QObject *target)
{
    if (target == ui->pushButton_QRCode)
    {
        if (m_licenseInfo.machineCode.isEmpty())
        {
            m_showQRCode->setQRCode(tr("Machine code is enpty!"), tr("Scan QR code to get machine code"));
        }
        else
        {
            m_showQRCode->setQRCode(m_licenseInfo.machineCode, tr("Scan QR code to get machine code"));
        }
    }

    m_showQRCode->move(oPoint);
    m_showQRCode->show();
}

void ActiveGuide::displayMessageBox(const QString &title, const QString &message)
{
    m_dialog->setDialogInfo(message);
    m_dialog->show();
}

}  // namespace VA
